@@ -51,31 +51,36 @@ class GeneralARInput(DataContainer):
         self.output_file = "atomicrex.out"
         self.parameter_constraints = ParameterConstraints()
 
-    def _write_xml_file(self, directory):
+    def _write_xml_file(self, directory, job=None):
         """Internal function.
         Write the main input xml file in a directory.
 
         Args:
             directory (str): Working directory
         """
-        job = ET.Element("job")
+        root = ET.Element("job")
 
-        output_file = ET.SubElement(job, "output-file")
+        output_file = ET.SubElement(root, "output-file")
         output_file.text = self.output_file
 
-        name = ET.SubElement(job, "name")
+        name = ET.SubElement(root, "name")
         name.text = self.name
 
-        verbosity = ET.SubElement(job, "verbosity")
+        verbosity = ET.SubElement(root, "verbosity")
         verbosity.text = self.verbosity
 
         if self.validate_potentials:
-            validate_potentials = ET.SubElement(job, "validate-potentials")
+            validate_potentials = ET.SubElement(root, "validate-potentials")
 
-        real_precision = ET.SubElement(job, "real-precision")
+        real_precision = ET.SubElement(root, "real-precision")
         real_precision.text = f"{self.real_precision}"
 
-        atom_types = ET.SubElement(job, "atom-types")
+        atom_types = ET.SubElement(root, "atom-types")
+        if len(self.atom_types) == 0:
+            eles = job.structures._structures.get_elements()
+            for ele in eles:
+                self.atom_types[ele] = None
+
         for k, v in self.atom_types.items():
             species = ET.SubElement(atom_types, "species")
             species.text = k
@@ -89,7 +94,7 @@ class GeneralARInput(DataContainer):
             species.set("atomic-number", f"{index}")
 
         if not isinstance(self.fit_algorithm, ScipyAlgorithm):
-            fitting = ET.SubElement(job, "fitting")
+            fitting = ET.SubElement(root, "fitting")
             if self.enable_fitting:
                 fitting.set("enabled", "true")
             else:
@@ -97,21 +102,21 @@ class GeneralARInput(DataContainer):
             fitting.set("output-interval", f"{self.output_interval}")
             fitting.append(self.fit_algorithm._to_xml_element())
 
-        potentials = ET.SubElement(job, "potentials")
+        potentials = ET.SubElement(root, "potentials")
         include = ET.SubElement(potentials, "xi:include")
         include.set("href", "potential.xml")
         include.set("xmlns:xi", "http://www.w3.org/2003/XInclude")
 
-        structures = ET.SubElement(job, "structures")
+        structures = ET.SubElement(root, "structures")
         include = ET.SubElement(structures, "xi:include")
         include.set("href", "structures.xml")
         include.set("xmlns:xi", "http://www.w3.org/2003/XInclude")
 
         if len(self.parameter_constraints) > 0:
-            job.append(self.parameter_constraints._to_xml_element())
+            root.append(self.parameter_constraints._to_xml_element())
 
         file_name = posixpath.join(directory, "main.xml")
-        write_pretty_xml(job, file_name)
+        write_pretty_xml(root, file_name)
 
 
 class AtomTypes(DataContainer):
@@ -527,6 +532,90 @@ class AlgorithmFactory(PyironFactory):
             seed=seed,
         )
 
+    @staticmethod
+    def g_mlsl(
+        stopval=1e-10,
+        max_iter=50,
+        maxtime=None,
+        ftol_rel=None,
+        ftol_abs=None,
+        xtol_rel=None,
+        seed=None,
+    ):
+        return NloptGlobalLocal(
+            name="G_MLSL",
+            stopval=stopval,
+            max_iter=max_iter,
+            maxtime=maxtime,
+            ftol_rel=ftol_rel,
+            ftol_abs=ftol_abs,
+            xtol_rel=xtol_rel,
+            seed=seed,
+        )
+
+    @staticmethod
+    def g_mlsl_lds(
+        stopval=1e-10,
+        max_iter=50,
+        maxtime=None,
+        ftol_rel=None,
+        ftol_abs=None,
+        xtol_rel=None,
+        seed=None,
+    ):
+        return NloptGlobalLocal(
+            name="G_MLSL_LDS",
+            stopval=stopval,
+            max_iter=max_iter,
+            maxtime=maxtime,
+            ftol_rel=ftol_rel,
+            ftol_abs=ftol_abs,
+            xtol_rel=xtol_rel,
+            seed=seed,
+        )
+
+    @staticmethod
+    def gd_stogo(
+        stopval=1e-10,
+        max_iter=50,
+        maxtime=None,
+        ftol_rel=None,
+        ftol_abs=None,
+        xtol_rel=None,
+        seed=None,
+    ):
+        return NloptAlgorithm(
+            name="GD_STOGO",
+            stopval=stopval,
+            max_iter=max_iter,
+            maxtime=maxtime,
+            ftol_rel=ftol_rel,
+            ftol_abs=ftol_abs,
+            xtol_rel=xtol_rel,
+            seed=seed,
+        )
+
+    @staticmethod
+    def gd_stogo_rand(
+        stopval=1e-10,
+        max_iter=50,
+        maxtime=None,
+        ftol_rel=None,
+        ftol_abs=None,
+        xtol_rel=None,
+        seed=None,
+    ):
+        return NloptAlgorithm(
+            name="GD_STOGO_RAND",
+            stopval=stopval,
+            max_iter=max_iter,
+            maxtime=maxtime,
+            ftol_rel=ftol_rel,
+            ftol_abs=ftol_abs,
+            xtol_rel=xtol_rel,
+            seed=seed,
+        )
+
     @staticmethod
     def scipy_algorithm():
         return ScipyAlgorithm()
@@ -565,17 +654,22 @@ class AtomicrexAlgorithm(DataContainer):
         return algo
 
 
-class SpaMinimizer:
+class SpaMinimizer(DataContainer):
     """
     Global optimizer implemented in atomicrex.
     Should be used in combination with a local minimizer.
     See the atomicrex documentation for details.
     """
 
-    def __init__(self, spa_iterations, seed):
-        self.spa_iterations = spa_iterations
+    def __init__(self, max_iter=None, seed=None, *args, **kwargs):
+        super().__init__(table_name="fitting_algorithm", *args, **kwargs)
+        self.max_iter = max_iter
         self.seed = seed
         self.local_minimizer = None
+    
+    @property
+    def name(self):
+        return "spa"
 
     def _to_xml_element(self):
         """Internal function.
@@ -583,10 +677,12 @@ class SpaMinimizer:
         and returns it
         """
         spa = ET.Element("spa")
-        spa.set("max-iter", f"{self.spa_iterations}")
+        spa.set("max-iter", f"{self.max_iter}")
         spa.set("seed", f"{self.seed}")
         if self.local_minimizer is not None:
             spa.append(self.local_minimizer._to_xml_element())
+        else:
+            raise ValueError("Set a local minimizer for Spa")
         return spa
 
 
@@ -637,6 +733,8 @@ class NloptAlgorithm(DataContainer):
             nlopt.set("ftol_abs", f"{self.ftol_abs}")
         if self.xtol_rel is not None:
             nlopt.set("xtol_rel", f"{self.xtol_rel}")
+        if self.seed is not None:
+            nlopt.set("seed", f"{self.seed}")
         return nlopt
 
 
@@ -647,14 +745,14 @@ class NloptGlobalLocal(NloptAlgorithm):
 
     def __init__(
         self,
-        stopval,
-        max_iter,
-        maxtime,
-        ftol_rel,
-        ftol_abs,
-        xtol_rel,
-        name,
-        seed,
+        stopval=None,
+        max_iter=None,
+        maxtime=None,
+        ftol_rel=None,
+        ftol_abs=None,
+        xtol_rel=None,
+        name=None,
+        seed=None,
         *args,
         **kwargs,
     ):
@@ -706,12 +804,32 @@ class ScipyAlgorithm:
         with hdf.open(group_name) as h:
             self._type_to_hdf(h)
             h["global_minimizer"] = self.global_minimizer
-            h.put("local_minimizer_kwargs", self.local_minimizer_kwargs)
-            h.put("global_minimizer_kwargs", self.global_minimizer_kwargs)
+        """
+            with h.open("local_minimizer_kwargs") as loc_hdf:
+                for k, v in self.local_minimizer_kwargs.items():
+                    try:
+                        loc_hdf[k] = v
+                    except TypeError:
+                        loc_hdf[k] = v.__name__
+            with h.open("global_minimizer_kwargs") as glob_hdf:
+                for k, v in self.global_minimizer_kwargs.items():
+                    if isinstance(v, dict):
+                        with glob_hdf.open(k) as v_hdf:
+                            for k, v in self.v.items():
+                                try:
+                                    v_hdf[k] = v
+                                except TypeError:
+                                    v_hdf[k] = v.__name__
+                    else:           
+                        try:
+                            glob_hdf[k] = v
+                        except TypeError:
+                            glob_hdf[k] = v.__name__
+        """
 
     def from_hdf(self, hdf, group_name):
         with hdf.open(group_name) as h:
-            self._type_to_hdf(h)
+            #self._type_from_hdf(h)
             self.global_minimizer = h["global_minimizer"]
 
     def _type_to_hdf(self, hdf):
