@@ -14,7 +14,7 @@ Let's make a structure and invent some forces
 
 >>> structure = pr.create.structure.ase_bulk("Fe")
 >>> forces = numpy.array([-1, 1, -1])
->>> container.include_structure(structure, energy=-1.234, forces=forces, name="Fe_bcc")
+>>> container.add_structure(structure, energy=-1.234, forces=forces, identifier="Fe_bcc")
 
 If you have a lot of precomputed structures you may also add them in bulk from a pandas DataFrame
 
@@ -99,7 +99,7 @@ class TrainingContainer(GenericJob, HasStructure):
                                           **properties)
 
     def add_structure(
-        self, structure, energy, forces=None, stress=None, name=None, **arrays
+        self, structure, energy, forces=None, stress=None, identifier=None, **arrays
     ):
         """
         Add new structure to structure list and save energy and forces with it.
@@ -115,7 +115,7 @@ class TrainingContainer(GenericJob, HasStructure):
             name (str, optional): name describing the structure
         """
         self._container.add_structure(
-            structure, energy, name=name, forces=forces, stress=stress, **arrays
+            structure, energy, identifier=identifier, forces=forces, stress=stress, **arrays
         )
 
     def include_dataset(self, dataset):
@@ -238,7 +238,8 @@ class TrainingContainer(GenericJob, HasStructure):
                 self.input.from_hdf(self.project_hdf5, "parameters")
 
     def sample(
-        self, name: str, selector: Callable[[StructureStorage, int], bool]
+        self, name: str, selector: Callable[[StructureStorage, int], bool],
+        delete_existing_job: bool = False
     ) -> "TrainingContainer":
         """
         Create a new TrainingContainer with structures filtered by selector.
@@ -250,6 +251,7 @@ class TrainingContainer(GenericJob, HasStructure):
         Args:
             name (str): name of the new TrainingContainer
             selector (Callable[[StructureStorage, int], bool]): callable that selects structure to include
+            delete_existing_job (bool): if job with name exist, remove it first
 
         Returns:
             :class:`.TrainingContainer`: new container with selected structures
@@ -257,7 +259,9 @@ class TrainingContainer(GenericJob, HasStructure):
         Raises:
             ValueError: if a job with the given `name` already exists.
         """
-        cont = self.project.create.job.TrainingContainer(name)
+        if not self.status.finished:
+            raise ValueError(f"Job must be finished, not '{self.status}'!")
+        cont = self.project.create.job.TrainingContainer(name, delete_existing_job=delete_existing_job)
         if not cont.status.initialized:
             raise ValueError(f"Job '{name}' already exists with status: {cont.status}!")
         cont._container = self._container.sample(selector)
@@ -551,11 +555,6 @@ class TrainingStorage(StructureStorage):
     def __init__(self):
         super().__init__()
         self.add_array("energy", dtype=np.float64, per="chunk", fill=np.nan)
-        self.add_array("forces", shape=(3,), dtype=np.float64, per="element",
-                       fill=np.nan)
-        # save stress in voigt notation
-        self.add_array("stress", shape=(6,), dtype=np.float64, per="chunk",
-                       fill=np.nan)
         self._table_cache = None
         self.to_pandas()
 
@@ -580,10 +579,12 @@ class TrainingStorage(StructureStorage):
                     "name": [self.get_array("identifier", i) for i in range(len(self))],
                     "atoms": [self.get_structure(i) for i in range(len(self))],
                     "energy": [self.get_array("energy", i) for i in range(len(self))],
-                    "forces": [self.get_array("forces", i) for i in range(len(self))],
-                    "stress": [self.get_array("stress", i) for i in range(len(self))],
                 }
             )
+            if self.has_array("forces"):
+                self._table_cache["forces"] = [self.get_array("forces", i) for i in range(len(self))]
+            if self.has_array("stress"):
+                self._table_cache["stress"] = [self.get_array("stress", i) for i in range(len(self))]
             self._table_cache["number_of_atoms"] = [
                 len(s) for s in self._table_cache.atoms
             ]
@@ -664,6 +665,13 @@ class TrainingStorage(StructureStorage):
         identifier=None,
         **arrays
     ) -> None:
+        if "forces" in arrays and not self.has_array("forces"):
+            self.add_array("forces", shape=(3,), dtype=np.float64, per="element",
+                           fill=np.nan)
+        if "stress" in arrays and not self.has_array("stress"):
+            # save stress in voigt notation
+            self.add_array("stress", shape=(6,), dtype=np.float64, per="chunk",
+                           fill=np.nan)
         super().add_structure(structure, identifier=identifier, energy=energy,
                               **arrays)
 
@@ -712,6 +720,8 @@ class TrainingStorage(StructureStorage):
             data_table = filter_function(data_table)
         structure_list = data_table.atoms.to_list()
         energy_list = data_table.energy.to_list()
+        if "forces" not in data_table.columns:
+            raise ValueError("no forces defined in storage; call to_dict() instead.")
         force_list = data_table.forces.to_list()
         num_atoms_list = data_table.number_of_atoms.to_list()
 
