@@ -3,9 +3,12 @@
 # Distributed under the terms of "New BSD License", see the LICENSE file.
 
 import os
-from pyiron.lammps.base import Input
-from pyiron.lammps.interactive import LammpsInteractive
+from pyiron_atomistics.lammps.base import Input
+from pyiron_atomistics.lammps.interactive import LammpsInteractive
+from pyiron_atomistics.atomistics.structure.atoms import Atoms
+from pyiron_atomistics.atomistics.structure.structurestorage import StructureStorage
 from pyiron_contrib.atomistics.mlip.mlip import read_cgfs
+from pyiron_contrib.atomistics.mlip.cfgs import loadcfgs
 from pyiron_base import GenericParameters
 
 __author__ = "Jan Janssen"
@@ -26,6 +29,7 @@ class LammpsMlip(LammpsInteractive):
         self.__version__ = None  # Reset the version number to the executable is set automatically
         self._executable = None
         self._executable_activate()
+        self._selected_structures = None
 
     def set_input_to_read_only(self):
         """
@@ -41,24 +45,70 @@ class LammpsMlip(LammpsInteractive):
             self.input.mlip['mtp-filename'] = os.path.basename(self.potential['Filename'][0][0])
         self.input.mlip.write_file(file_name="mlip.ini", cwd=self.working_directory)
 
-    def enable_active_learning(self):
-        self.input.mlip.load_string("""\
+    def convergence_check(self):
+        for line in self["error.out"]:
+            if line.startswith("MLIP: Breaking threshold exceeded"): return False
+        return True
+
+    def enable_active_learning(self, threshold=2.0, threshold_break=5.0):
+        """
+        Enable active learning during MD run.
+
+        Automatically collect structures on which the potential is extrapolating.
+
+        Args:
+            threshold (float): select structures with extrapolation grade larger than this
+            threshold_break (float): stop the MD run after seeing a structure with extrapolation grade larger than this
+        """
+        self.executable.accepted_return_codes += [8]
+        self.input.mlip.load_string(f"""\
 mtp-filename auto
 calculate-efs TRUE
 select TRUE
-select:threshold 2.0
-select:threshold-break 5.0
+select:threshold {threshold}
+select:threshold-break {threshold_break}
 select:save-selected selected.cfg
 select:load-state state.mvs
 select:log selection.log
 write-cfgs:skip 0
 """)
 
+    def _get_selection_file(self):
+        return os.path.join(self.working_directory, self.input.mlip['select:save-selected'])
+
+    @property
+    def _selection_enabled(self):
+        return self.input.mlip["select"] == "TRUE"
+
+    @property
+    def selected_structures(self):
+        """
+        :class:`.StructureStorage`: structures that the potential extrapolated on during the run.
+
+        Only available if :method:`.enable_active_learning` was called and once the job has been collected.
+        """
+        if not (self.status.collect or self.status.finished or self.status.not_converged):
+            raise ValueError("Selected structures are only available once the job has finished!")
+        if not self._selection_enabled:
+            raise ValueError("Selected structures are only available after calling enable_active_learning()!")
+        if self._selected_structures is None:
+            if "selected" in self["output"].list_groups():
+                self._selected_structures = self["output/selected"].to_object()
+            else:
+                self._selected_structures = StructureStorage()
+        return self._selected_structures
+
     def collect_output(self):
         super(LammpsMlip, self).collect_output()
         if 'select:save-selected' in self.input.mlip._dataset['Parameter']:
-            file_name = os.path.join(self.working_directory, self.input.mlip['select:save-selected'])
+            file_name = self._get_selection_file()
             if os.path.exists(file_name):
+                for cfg in loadcfgs(file_name):
+                    self.selected_structures.add_structure(
+                            Atoms(species=self.structure.species, indices=cfg.types, positions=cfg.pos, cell=cfg.lat,
+                                  pbc=[True, True, True]),
+                            mv_grade=cfg.grade
+                    )
                 cell, positions, forces, stress, energy, indicies, grades, jobids, timesteps = read_cgfs(file_name=file_name)
                 with self.project_hdf5.open("output/mlip") as hdf5_output:
                     hdf5_output['forces'] = forces
@@ -67,6 +117,7 @@ write-cfgs:skip 0
                     hdf5_output['cells'] = cell
                     hdf5_output['positions'] = positions
                     hdf5_output['indicies'] = indicies
+            self.selected_structures.to_hdf(self.project_hdf5.open("output"), "selected")
 
 
 class MlipInput(Input):
